%%Author: Pallavi A Raiturkar
%%function to clean up eyetracking data using velocity threshold of
%%5.65mm/s, and removing any blink train samples

function [eyeTrackingData] = cleanEyeTrackingData(eyeTrackingData)

%remove samples that had gaze position outside monitor dimensions
[eyeTrackingData] = correctEyeTrackingData(eyeTrackingData);
% size(eyeTrackingEvents)
% eyeTrackingData
%save original time scale (to be used later while interpolating)
time_original = eyeTrackingData.time;

%compute velocity of each sample (set velocity of first sample as ZERO)
velocityPupil = diff(eyeTrackingData.pupil)./diff(eyeTrackingData.time*1.0e-6);
velocityPupil = [0;velocityPupil];

%remove samples with higher velocity than 5.65mm/s or if PD = 0 for a sample
eyeTrackingData.pupil(abs(velocityPupil)>5.65 | eyeTrackingData.pupil==0) = NaN;

%find indices of all elements that are NaN in eyeTrackingData.pupil
missingSampleIndices = find(isnan(eyeTrackingData.pupil));

%compute the number of samples between two NaN values ( we want to ignore
%1,2 or 3 samples between NaNs because they're possibly blinks)

diffSamples = diff(missingSampleIndices);
samplesBetweenMissingSamples = [0; diffSamples]; %assume diff(1) = 0 (since it can't be computed)

thresholdBlink = 10; %up to 10 samples between NaNs will be removed

for v = 1:size(missingSampleIndices,1)
    for i = 2:(thresholdBlink+1)
        if(samplesBetweenMissingSamples(v) == i)
           for j = 1:i-1
               eyeTrackingData.pupil(missingSampleIndices(v)-j) = NaN; 
           end
        end
    end
end

missingSampleIndices = find(isnan(eyeTrackingData.pupil));

%remove all NaN samples
eyeTrackingData.pupil(missingSampleIndices) = [];

%remove same samples from time index
eyeTrackingData.time(missingSampleIndices) = [];
%interpolate missing samples
pupil = interp1(eyeTrackingData.time,eyeTrackingData.pupil,time_original);

eyeTrackingData.pupil = pupil;
eyeTrackingData.time = time_original;

end