%Script shared by Dr. Hany Farid
%Name: computeSensitivityBias
% PHIT is the proportion of "Hits":        P(Yes|Signal)
% PFA is the proportion of "False Alarms": P(Yes|Noise)
% PHIT and PFA are bounded by [0,1]
%
% N is the total number of conditions from which PHIT and PFA are estimated (assumed to be equal)
 
function [d,beta] = computeSensitivityBias(pHit,pFA,N)
 
% deal with pHit/pFA values of 0 or 1.
if pFA <= 0
    pfalse = 1/N;
elseif pFA >= 1
    pfalse = (N-1)/N;
else
    pfalse = pFA;
end
 
if pHit >= 1
    phit = (N-1)/N;
elseif pHit <= 0
    phit = 1/N;
else
    phit = pHit;
end
 
% Convert to Z-scores
zHit = norminv(phit);
zFA  = norminv(pfalse);
 
% Calculate d-prime
d = zHit - zFA;
 
% Calculate beta
yHit = normpdf(zHit);
yFA  = normpdf(zFA);
beta = yHit ./ yFA;
end