function [Time,Type,Trial,LPupilDiametermm,RPupilDiametermm,LPORXpx,LPORYpx,RPORXpx,RPORYpx,LEventInfo,REventInfo,Stimulus] = importfile(filename, startRow, endRow)
%IMPORTFILE Import numeric data from a text file as column vectors.
%   [TIME,TYPE,TRIAL,LPUPILDIAMETERMM,RPUPILDIAMETERMM,LPORXPX,LPORYPX,RPORXPX,RPORYPX,LEVENTINFO,REVENTINFO,STIMULUS]
%   = IMPORTFILE(FILENAME) Reads data from text file FILENAME for the
%   default selection.
%
%   [TIME,TYPE,TRIAL,LPUPILDIAMETERMM,RPUPILDIAMETERMM,LPORXPX,LPORYPX,RPORXPX,RPORYPX,LEVENTINFO,REVENTINFO,STIMULUS]
%   = IMPORTFILE(FILENAME, STARTROW, ENDROW) Reads data from rows STARTROW
%   through ENDROW of text file FILENAME.
%
% Example:
%   [Time,Type,Trial,LPupilDiametermm,RPupilDiametermm,LPORXpx,LPORYpx,RPORXpx,RPORYpx,LEventInfo,REventInfo,Stimulus] = importfile('2017_eyetracking_cgfaces_p001_001_Trial011 Samples.txt',2, 709);
%
%    See also TEXTSCAN.

% Auto-generated by MATLAB on 2017/10/11 17:09:22

%% Initialize variables.
delimiter = ',';
if nargin<=2
    startRow = 2;
    endRow = inf;
end

%% Read columns of data as strings:
% For more information, see the TEXTSCAN documentation.
formatSpec = '%s%s%s%s%s%s%s%s%s%s%s%s%[^\n\r]';

%% Open the text file.
fileID = fopen(filename,'r');

%% Read columns of data according to format string.
% This call is based on the structure of the file used to generate this
% code. If an error occurs for a different file, try regenerating the code
% from the Import Tool.
dataArray = textscan(fileID, formatSpec, endRow(1)-startRow(1)+1, 'Delimiter', delimiter, 'HeaderLines', startRow(1)-1, 'ReturnOnError', false);
for block=2:length(startRow)
    frewind(fileID);
    dataArrayBlock = textscan(fileID, formatSpec, endRow(block)-startRow(block)+1, 'Delimiter', delimiter, 'HeaderLines', startRow(block)-1, 'ReturnOnError', false);
    for col=1:length(dataArray)
        dataArray{col} = [dataArray{col};dataArrayBlock{col}];
    end
end

%% Close the text file.
fclose(fileID);

%% Convert the contents of columns containing numeric strings to numbers.
% Replace non-numeric strings with NaN.
raw = repmat({''},length(dataArray{1}),length(dataArray)-1);
for col=1:length(dataArray)-1
    raw(1:length(dataArray{col}),col) = dataArray{col};
end
numericData = NaN(size(dataArray{1},1),size(dataArray,2));

for col=[1,3,4,5,6,7,8,9,12]
    % Converts strings in the input cell array to numbers. Replaced non-numeric
    % strings with NaN.
    rawData = dataArray{col};
    for row=1:size(rawData, 1);
        % Create a regular expression to detect and remove non-numeric prefixes and
        % suffixes.
        regexstr = '(?<prefix>.*?)(?<numbers>([-]*(\d+[\,]*)+[\.]{0,1}\d*[eEdD]{0,1}[-+]*\d*[i]{0,1})|([-]*(\d+[\,]*)*[\.]{1,1}\d+[eEdD]{0,1}[-+]*\d*[i]{0,1}))(?<suffix>.*)';
        try
            result = regexp(rawData{row}, regexstr, 'names');
            numbers = result.numbers;
            
            % Detected commas in non-thousand locations.
            invalidThousandsSeparator = false;
            if any(numbers==',');
                thousandsRegExp = '^\d+?(\,\d{3})*\.{0,1}\d*$';
                if isempty(regexp(numbers, thousandsRegExp, 'once'));
                    numbers = NaN;
                    invalidThousandsSeparator = true;
                end
            end
            % Convert numeric strings to numbers.
            if ~invalidThousandsSeparator;
                numbers = textscan(strrep(numbers, ',', ''), '%f');
                numericData(row, col) = numbers{1};
                raw{row, col} = numbers{1};
            end
        catch me
        end
    end
end


%% Split data into numeric and cell columns.
rawNumericColumns = raw(:, [1,3,4,5,6,7,8,9,12]);
rawCellColumns = raw(:, [2,10,11]);


%% Allocate imported array to column variable names
Time = cell2mat(rawNumericColumns(:, 1));
Type = rawCellColumns(:, 1);
Trial = cell2mat(rawNumericColumns(:, 2));
LPupilDiametermm = cell2mat(rawNumericColumns(:, 3));
RPupilDiametermm = cell2mat(rawNumericColumns(:, 4));
LPORXpx = cell2mat(rawNumericColumns(:, 5));
LPORYpx = cell2mat(rawNumericColumns(:, 6));
RPORXpx = cell2mat(rawNumericColumns(:, 7));
RPORYpx = cell2mat(rawNumericColumns(:, 8));
LEventInfo = rawCellColumns(:, 2);
REventInfo = rawCellColumns(:, 3);
Stimulus = cell2mat(rawNumericColumns(:, 9));


