%%Author: Pallavi A Raiturkar
%Name: readEyeTrackingData
%Parameters:filepath
%Output:eyeTrackingData (struct with fields time, x, y, pupil)
%%
%This is a wrapper function that accepts the path of the file which contains the raw eye tracking 
%data(filepath) and and converts the data into a MATLAB structure with fields
%x, y, time and pupil, which store the x coordinate, y coordinate,
%timestamp and pupil diameter of the eye tracking data points respectively.
%It returns this structure to the variable assigned to the function call.
function [eyeTrackingData]=readEyeTrackingData(filepath)
	%file format by default is .txt
    %[pathstr,name,ext]=fileparts(filepath)
	[~,~,ext] = fileparts(filepath);
	%check file format
	switch ext
		case '.txt'
            [Time,Type,Trial,LPupilDiametermm,LPORXpx,LPORYpx,Stimulus]=importfile(filepath);
% 			formatSpec = '%f%s%f%f%f%f%s'; %to get all 4 values as floating point
% 			eyeTrackingData1 = readtable(filepath,'Delimiter',',', ...
%                 'Format',formatSpec);
% 			eyeTrackingData1.Properties.VariableNames{'Time'}='time';
%             eyeTrackingData1.Properties.VariableNames{'LPupilDiameter_mm_'}='pupil';
%             eyeTrackingData1.Properties.VariableNames{'LPORX_px_'}='x';
%             eyeTrackingData1.Properties.VariableNames{'LPORY_px_'}='y';
%             eyeTrackingData=struct('x',num2cell(LPORXpx),'y',num2cell(LPORYpx),'time',num2cell(Time),'pupil',num2cell(LPupilDiametermm),'event',LEventInfo);
            eyeTrackingData=struct('x',LPORXpx(2:size(LPORXpx)),'y',LPORYpx(2:size(LPORXpx)),'time',Time(2:size(LPORXpx)),'pupil',LPupilDiametermm(2:size(LPORXpx)));
%             eyeTrackingEvents = LEventInfo(2:size(LPORXpx));
            eyeTrackingData.time=eyeTrackingData.time-eyeTrackingData.time(1);
		case '.mat'
			tempStruct=load(filepath);
            tempSubject=char(fieldnames(tempStruct));
			value1=tempStruct.(tempSubject).gaze.x; 
			value2=tempStruct.(tempSubject).gaze.y;
			value3=tempStruct.(tempSubject).gaze.timestamp;
            value4=tempStruct.(tempSubject).gaze.pupil;
			eyeTrackingData=struct('x',value1,'y',value2,'time',value3,'pupil',value4);
            eyeTrackingData.time=eyeTrackingData.time-eyeTrackingData.time(1);
        case '.xlsx'
            tempData=xlsread(filepath); %will contain matrix with all the data
            %stimuliID=tempData(1,1); %all values in column 1 are the same, take any
            value1=tempData(:,2); %time
			value2=tempData(:,3); %x value
			value3=tempData(:,4); %y value
            value4=tempData(:,5); %pupil diameter
            eyeTrackingData=struct('x',value2,'y',value3,'time',value1,'pupil',value4);
            eyeTrackingData.time=eyeTrackingData.time-eyeTrackingData.time(1);
        otherwise
            %if file format is not recognized
			error('Error: Unrecognized file format')
	end

end