%%Author: Pallavi A Raiturkar
%%
%function to return fixation points (low velocity points), saccade
%points(high velocity points), fixation struct, blink struct and saccade
%struct
%%
function [eyeTrackingDistance,eyeTrackingVelocity,eyeTrackingDataLabel,fixation,saccade,fixationPoints,saccadePoints]=returnFixationSaccade(eyeTrackingData)

    numberDataPoints=size(eyeTrackingData.time,1);

    %%set velocity threshold for fixation detection using I-VT
    threshold=130; %angular velocity threshold in degrees/second
    screen_vertical_resolution=1050; %vertical screen resolution
    screen_physical_height=29; %vertical height of screen in cm
    distance_eye=64; %distance of screen from eye in cm
    v_threshold=degtopix(screen_vertical_resolution,screen_physical_height,distance_eye,threshold); %threshold in pixels

    %%sort through all data points to classify into fixation points and saccade points, and store indices and group number
    %initialisation
    k=1; 
    j=1;
    eyeTrackingDataLabel=zeros(1,1);
    fixationPointIndex=zeros(1,1);
    fixationPointGroup=1;
    saccadePointIndex=zeros(1,1);
    eyeTrackingDistance=zeros(numberDataPoints-1,1);
    eyeTrackingVelocity=zeros(numberDataPoints-1,1);
    fixationTimeEnd=zeros(1,1);
    %end_initialisation
    saccadeFlag=0;
    for i=1:numberDataPoints-1
        eyeTrackingDistance(i)=sqrt((eyeTrackingData.x(i+1)-eyeTrackingData.x(i))^2+(eyeTrackingData.y(i+1)-eyeTrackingData.y(i))^2); %distance between successive pixel coordinates
        eyeTrackingVelocity(i)=(eyeTrackingDistance(i)*1.0e06)/(eyeTrackingData.time(i+1)-eyeTrackingData.time(i));  %point to point velocity
        if(eyeTrackingVelocity(i)<v_threshold) %fixation point
            eyeTrackingDataLabel(i,:)=1;
            fixationPointIndex(k,1)=i; %indices of all low velocity points
            fixationPointIndex(k,2)=fixationPointGroup;
            k=k+1; 
            saccadeFlag=1;
        else %saccade point
            if saccadeFlag==1
                fixationTimeEnd(fixationPointGroup,:)=eyeTrackingData.time(i);
                fixationPointGroup=fixationPointGroup+1;
            end
    %         plot(i,eyeTrackingVelocity(i),'rx');
    %         hold on;
            eyeTrackingDataLabel(i,:)=0;
            saccadePointIndex(j,1)=i;%indices of all high velocity points
            j=j+1;
            saccadeFlag=0;
        end
    end
    if saccadeFlag==1
        fixationTimeEnd(fixationPointGroup,:)=eyeTrackingData.time(i+1); % store end time of last fixation group
    end


    %%store fixation and saccade points in separate MATLAB structures
    fixationPoints=struct('x',[],'y',[],'time',[],'pupil',[],'group',[]);
    saccadePoints=struct('x',[],'y',[],'time',[],'pupil',[]);

    fixationPoints.x=eyeTrackingData.x(fixationPointIndex(:,1));
    fixationPoints.y=eyeTrackingData.y(fixationPointIndex(:,1));
    fixationPoints.time=eyeTrackingData.time(fixationPointIndex(:,1));
    fixationPoints.pupil=eyeTrackingData.pupil(fixationPointIndex(:,1));
    fixationPoints.group=fixationPointIndex(:,2);
    
    if (saccadePointIndex~=0)
    saccadePoints.x=eyeTrackingData.x(saccadePointIndex(:,1));
    saccadePoints.y=eyeTrackingData.y(saccadePointIndex(:,1));
    saccadePoints.time=eyeTrackingData.time(saccadePointIndex(:,1));
    saccadePoints.pupil=eyeTrackingData.pupil(saccadePointIndex(:,1));
    end
    %%find centroids of each fixation group, its start time and duration

    %%
    numberFixationPoints=size(fixationPointIndex,1);
    k=1; i=1;
    
    fixation=struct('index',[],'x',[],'y',[],'starttime',[],'endtime',[],'duration',[]);
    saccade=struct('index',[],'starttime',[],'length',[],'window',[]);

    while i<=numberFixationPoints-1 
        n=0; 
        sum_x=0;
        sum_y=0;
        while(i<=numberFixationPoints-1 && fixationPoints.group(i)==k)
           n=n+1; %current number of elements in fixation group
           sum_x=sum_x+fixationPoints.x(i);
           sum_y=sum_y+fixationPoints.y(i);
           i=i+1;
        end
        fixation.index(k,:)=k;
        fixation.x(k,:)=sum_x/n; %x of centroid 
        fixation.y(k,:)=sum_y/n; %y of centroid 
        fixation.starttime(k,:)=fixationPoints.time(i-n); %start time of fixation
        k=k+1; %to increment index of fixation group
    end
    
    %to ensure consistent number of fixations
    numberFixations=size(fixation.starttime,1);
    fixation.endtime=fixationTimeEnd(1:numberFixations);
    fixation.duration=fixation.endtime-fixation.starttime;
    
    %%find saccade lengths as distance between consecutive fixations, and the 
    %corresponding start time of each saccade
    if(saccadePointIndex~=0)
    for i=1:numberFixations-1
        saccade.index(i,:)=i;
        saccade.length(i,:)=sqrt((fixation.x(i+1)-fixation.x(i)).^2+(fixation.y(i+1)-fixation.y(i)).^2); 
        saccade.starttime(i,:)=fixation.endtime(i); %end time of fixation marks beginning of a saccade
    end
    end
end
